from pcpbase.pcpbaseDefaultInputs import defaultpcpBaseInputs
from theia import Theia
from theia.aws.network import Network, NetworkOptions, NetworkSubnet
from typing import Literal, Any, Optional, Mapping

class pcpNetworkSubnet:

    def __init__(self, userInput: dict):
        self.cidr = userInput['cidr_block']
        self.type = userInput['type']
        self.az = userInput['az']

    def theiaNetworkSubnet(self):
        subnet = NetworkSubnet(cidr_block=self.cidr,
                               type=self.type,
                               availability_zone=self.az)
        return subnet

class pcpNetwork:

    @staticmethod
    def getpcpVpcSubnets(userInputs: dict, 
                         defaultVpc: bool,
                         defaultInputs: dict = defaultpcpBaseInputs):
        
        networkVpcSubnets = []
        if defaultVpc:
            defaultSubnets = defaultInputs['vpc_subnets']
            for subnet in defaultSubnets:
                vpcSubnet = pcpNetworkSubnet(subnet)
                networkVpcSubnets.append(vpcSubnet)
        else:
            userSubnets = userInputs['vpc_subnets']
            for subnet in userSubnets:
                vpcSubnet = pcpNetworkSubnet(subnet)
                networkVpcSubnets.append(vpcSubnet)
        return networkVpcSubnets
            

            
    def __init__(self,
                 userInputs: dict,
                 defaultInputs: dict = defaultpcpBaseInputs):
        
        self.defaultVpc = True

        self.networkName = pcpBase.getpcpBaseResourceName(userInput=userInputs)['NetworkName']
        self.vpc_cidr = defaultInputs['vpc_cidr']

        if "vpc_cidr" in userInputs:
            self.defaultVpc = False
            self.vpc_cidr = userInputs['vpc_cidr']
        self.vpc_subnets = pcpNetwork.getpcpVpcSubnets(userInputs=userInputs, defaultVpc=self.defaultVpc)
        
        self.creteNatGateway = defaultInputs['create_nat_gateway']
        if 'create_nat_gateway' in userInputs:
            self.creteNatGateway = userInputs['create_nat_gateway']
        
        self.creteIGW = defaultInputs['create_igw']
        if 'create_igw' in userInputs:
            self.creteIGW = userInputs['create_igw']


class pcpBase:

    @staticmethod
    def getpcpBaseResourceName(userInput: dict):
        resourceList = {}
        baseStack = pcpBase.getpcpBaseStackName(userInputs=userInput)
        network = {
            "NetworkName" : f"{baseStack}-network"
        }
        resourceList.update(network)
        return resourceList

    @staticmethod 
    def getpcpBaseStackName(userInputs: dict):
        return f"{userInputs['project']}-{userInputs['env']}-{pcpBase.__name__}"

    def __init__(self,
                 userInputs: dict,
                 defaultInputs: dict = defaultpcpBaseInputs):
        
        self.userInputs = userInputs
        self.defaultInputs = defaultInputs

        self.project = f"{self.userInputs['project']}-{self.userInputs['env']}"
        self.email = self.userInputs['email']
        self.backend_url = self.userInputs['backend_url']
        self.region = self.userInputs['region']

        self.stack = pcpBase.__name__
        
        self.defaultInputs['tags']["Stack"] = pcpBase.getpcpBaseStackName(self.userInputs)
        self.defaultInputs['tags']['created_by'] = self.email

        self.tags = self.defaultInputs['tags']
        if "tags" in self.userInputs:
            self.tags.update(self.userInputs['tags'])


    def infra(self):

        def program():

            networkInfra = pcpNetwork(self.userInputs)

            networkSpecs = []
            for subnet in networkInfra.vpc_subnets:
                theiaSubnet = subnet.theiaNetworkSubnet()
                networkSpecs.append(theiaSubnet)
            
            networkOpts = NetworkOptions(create_nat_gateway=networkInfra.creteNatGateway,
                                        create_internet_gateway=networkInfra.creteIGW,
                                        create_vpc_security_group=True)
            

            theiaNetworkInfra = Network(resource_name=pcpBase.getpcpBaseResourceName(self.userInputs)['NetworkName'],
                                        vpc_cidr=networkInfra.vpc_cidr,
                                        subnet_specs=networkSpecs,
                                        tenancy="default",
                                        tags = self.tags,
                                        create_opts=networkOpts)
            
        t =  Theia(project=self.project,
                    stack=self.stack,
                    program=program,
                    region=self.region,
                    backend_url=self.backend_url)
        
        return t
    
    
    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        t = self.infra()
        getattr(t, command)()


        

userInputs = {
    "project" : "test",
    "env" : "dev",
    "email" : "ayush.kumar@amagi.com",
    "region" : "us-east-1",
    "backend_url" : "s3://pcp-as-code-draft-ayush",
    "tags" : {
    }
}

t1 = pcpBase(userInputs=userInputs)
# print(t1.userInputs)
t1('up')